@@ -288,39 +288,18 @@ where
         value.write_sized(self, length)
     }
 
-    /// Reserve some bits to be written later by splitting of two parts
-    ///
-    /// This allows skipping a few bits to write later
-    pub fn reserve<
-        Err,
-        F: Fn(&mut BitWriteStream<E>, &mut BitWriteStream<E>) -> Result<(), Err>,
-    >(
-        &mut self,
-        count: usize,
-        body_fn: F,
-    ) -> Result<(), Err> {
-        let (head, tail) = self.buffer.reserve(count);
-        let mut head = BitWriteStream { buffer: head };
-        let mut tail = BitWriteStream { buffer: tail };
-        body_fn(&mut head, &mut tail)
-    }
-
     /// Write the length of a section before the section
     pub fn reserve_length<Err: From<BitError>, F: Fn(&mut BitWriteStream<E>) -> Result<(), Err>>(
         &mut self,
         length_bit_size: usize,
         body_fn: F,
     ) -> Result<(), Err> {
-        let (head, tail) = self.buffer.reserve(length_bit_size);
-        let mut head = BitWriteStream { buffer: head };
-        let mut tail = BitWriteStream { buffer: tail };
-
-        let start = tail.bit_len();
-        body_fn(&mut tail)?;
-        let end = tail.bit_len();
-        let bit_len = end - start;
-        head.write_sized(&bit_len, length_bit_size)?;
-        Ok(())
+        self.reserve_int(length_bit_size, |stream| {
+            let start = stream.bit_len();
+            body_fn(stream)?;
+            let end = stream.bit_len();
+            Ok((end - start) as u64)
+        })
     }
 
     /// Write the length in bytes of a section before the section, the section will be 0 padded to an even byte length
@@ -332,36 +311,32 @@ where
         length_bit_size: usize,
         body_fn: F,
     ) -> Result<(), Err> {
-        let (head, tail) = self.buffer.reserve(length_bit_size);
-        let mut head = BitWriteStream { buffer: head };
-        let mut tail = BitWriteStream { buffer: tail };
+        self.reserve_int(length_bit_size, |stream| {
+            let start = stream.bit_len();
+            body_fn(stream)?;
+            let end = stream.bit_len();
+            let bit_len = end - start;
 
-        let start = tail.bit_len();
-        body_fn(&mut tail)?;
-        let end = tail.bit_len();
-        let bit_len = end - start;
+            let pad_len = (8 - (bit_len & 7)) & 7;
+            stream.push_bits(0, pad_len);
 
-        let pad_len = (8 - (bit_len & 7)) & 7;
-        tail.push_bits(0, pad_len);
-
-        let byte_len = (bit_len + pad_len) / 8;
-
-        head.write_sized(&byte_len, length_bit_size)?;
-        Ok(())
+            let byte_len = (bit_len + pad_len) / 8;
+            Ok(byte_len as u64)
+        })
     }
 
     /// Reserve the length to write an integer
     pub fn reserve_int<Err: From<BitError>, F: Fn(&mut BitWriteStream<E>) -> Result<u64, Err>>(
         &mut self,
-        length_bit_size: usize,
+        count: usize,
         body_fn: F,
     ) -> Result<(), Err> {
-        let (head, tail) = self.buffer.reserve(length_bit_size);
-        let mut head = BitWriteStream { buffer: head };
-        let mut tail = BitWriteStream { buffer: tail };
+        let start = self.bit_len();
+        self.write_int(0u64, count)?;
+
+        let head_int = body_fn(self)?;
+        self.buffer.set_at(start, head_int, count);
 
-        let head_int = body_fn(&mut tail)?;
-        head.write_sized(&head_int, length_bit_size)?;
         Ok(())
     }
 }
